#!/usr/bin/perl

use strict;
use POSIX;

my $data_fifo = "/tmp/data_";
my $img_fifo = "/tmp/img_";
srand;
for (my $i = 0; $i < 32; $i++) {
	$data_fifo .= chr(ord('a') + int(rand() * 26));
	$img_fifo .= chr(ord('a') + int(rand() * 26));
}

my $script = <<EOS;
set terminal pngcairo
set output "$img_fifo"
unset key
plot "$data_fifo"
exit
EOS

my $data = <<EOD;
1	1
2	5
3	10
4	7
5	8
6	3
7	2
8	9
9	4
10	6
EOD

mkfifo($data_fifo, 0600) or die "mkfifo error\n";
unless (mkfifo($img_fifo, 0600)) {
	unlink($data_fifo);
	die "mkfifo error\n";
}

my $pid1 = fork();
if ($pid1 == -1) {
	unlink($data_fifo);
	unlink($img_fifo);
	die "fork error\n";
}
if ($pid1 == 0) {
	open(DATA, "> $data_fifo") or die "data open error\n";
	print DATA $data;
	close(DATA);
	exit;
}

my $pid2 = fork();
if ($pid2 == -1) {
	unlink($data_fifo);
	unlink($img_fifo);
	die "fork error 2\n";
}
if ($pid2 == 0) {
	open(GP, "| gnuplot > /dev/null 2>&1") or die "gnuplot open error\n";
	print GP $script;
	close(GP);
	exit;
}

unless (open(IMG, "< $img_fifo")) {
	unlink($data_fifo);
	unlink($img_fifo);
	die "img open error\n";
}
binmode(IMG);
my $image_data = '';
while (<IMG>) { $image_data .= $_; }
close(IMG);

waitpid($pid1, 0);
waitpid($pid2, 0);

unlink($data_fifo);
unlink($img_fifo);

if (@ARGV > 0 && open(OUT, "> $ARGV[0]")) {
	binmode(OUT);
	print OUT $image_data;
	close(OUT);
} else {
	printf "success. got %d bytes.\n", length($image_data);
}

